#include "mex.h"
#include <stdio.h>
#include <math.h>

int* updateBuffer(int* B, int numInBuffer, int s, int t, int item, int randCoinFlip, int randLocation){
    /* If there is space just add it */
    if(numInBuffer < s){
        *(B + numInBuffer) = item;
    }/* Buffer is full - either replace an element with incoming element or let the buffer be as it is */
    else if(numInBuffer == s){
        if(randCoinFlip < s/t)
            *(B + randLocation) = item;
    }
    else{
        mexErrMsgTxt("Buffer in state of overflow !\n");
    }
    
    return B;
}

double* doOAMGra(double* XTrain, double* yTrain, int numTrain, int d, double C, int sPos, int sNeg, int EPos, int ENeg, double* randCoinFlips, double* randLocations){
    int t, dCounter, bufferCounter, y_t, runCounterPos, runCounterNeg, updateCounter, numPosInBuffer, numNegInBuffer, numPointsInOtherBuffer, numLosses;
    double C_t, temp;
    double *x_t, *w, *wBar, *xBuf_t, *del_t;
    int *BPos, *BNeg, *otherBuffer;
    char msg[200];
    int junk;
    
    /* Used to update the final weight vector after each update */
    updateCounter = 0;
    
    /* Run counters help us keep track of progress within an epoch */
	runCounterPos = 0;
	runCounterNeg = 0;
    
    /* Initialize the buffers to empty sets - buffers just contain the indices not the actual training points */
	BPos = (int*)malloc(sPos*sizeof(int));
	BNeg = (int*)malloc(sNeg*sizeof(int));
    
    for(bufferCounter = 0; bufferCounter < sPos; bufferCounter++){
        *(BPos + bufferCounter) = -1;
    }
    for(bufferCounter = 0; bufferCounter < sNeg; bufferCounter++){
        *(BNeg + bufferCounter) = -1;
    }
    
	numPosInBuffer = 0;
	numNegInBuffer = 0;
    
    C_t = 0;
    
    /* Intermediate weight vector */
	w = (double*)malloc(d*sizeof(double));
    
    /* Final weight vector */
    wBar = (double*)malloc(d*sizeof(double));
    
    /* Step vector */
    del_t = (double*)malloc(d*sizeof(double));
    
	/* Initially both weight vectors are zero */
    for(dCounter = 0; dCounter < d; dCounter++){
        *(w + dCounter) = 0;
        *(wBar + dCounter) = 0;
    }
    
    for(t = 0; t < numTrain; t++){
        x_t = XTrain + d*t;
        y_t = (int)*(yTrain + t);
		C_t = C/sqrt(t+1);
        
        if(y_t == 1){
            runCounterPos++;
            otherBuffer = BNeg;
            numPointsInOtherBuffer = numNegInBuffer;
        }
        else if(y_t == -1){
            runCounterNeg++;
            otherBuffer = BPos;
            numPointsInOtherBuffer = numPosInBuffer;
        }
        else{
            junk = sprintf(&(msg[0]),"Bad label at location %d\n",t+1);
            mexErrMsgTxt(msg);
        }
        
        if(numPointsInOtherBuffer > 0){
			/* We will definitely make an update (even if a zero one) at this time step */
			updateCounter++;
			
			/* On how many points did we incur a loss */
			numLosses = 0;
			
			/* Set the step vector to zero */
			for(dCounter = 0; dCounter < d; dCounter++){
				*(del_t + dCounter) = 0;
			}
			
        	for(bufferCounter = 0; bufferCounter < numPointsInOtherBuffer; bufferCounter++){
				/* Get the buffer point */
                xBuf_t = XTrain + d*(*(otherBuffer + bufferCounter));
				
				/* Find out if this pair incurs any non zero hinge loss */
				/* i.e. if y_t * w_t(x_t - x_buf) < 1 */
				temp = 0;
				for(dCounter = 0; dCounter < d; dCounter++){
					temp += (*(w + dCounter))*(*(x_t + dCounter) - *(xBuf_t + dCounter));
				}
				temp *= y_t;
				
				/* If we have incured a loss then update the step vector */
				if(temp <= 1){
					numLosses++;	
					for(dCounter = 0; dCounter < d; dCounter++){
						*(del_t + dCounter) += *(xBuf_t + dCounter);
					}
				}
            }
			
			/* Update the intermediate weight vector if required */
			if(numLosses > 0){
				for(dCounter = 0; dCounter < d; dCounter++){
					*(w + dCounter) += C_t*y_t/numPointsInOtherBuffer*(numLosses*(*(x_t + dCounter)) - *(del_t + dCounter));
				}
			}
			
			/* Update the final weight vector */
			for(dCounter = 0; dCounter < d; dCounter++){
				*(wBar + dCounter) = ((*(wBar + dCounter))*(updateCounter - 1) + *(w + dCounter))/updateCounter;
			}
        }
                
        /* Update the respective buffers */
		if(y_t == +1){
			/* Does the buffer need to be refreshed ? */
			if(runCounterPos > EPos){
				/* Refresh the buffer and start a new epoch */
                for(bufferCounter = 0; bufferCounter < sPos; bufferCounter++){
                    *(BPos + bufferCounter) = -1;
                }
				numPosInBuffer = 0;
				runCounterPos = 1;
            }
			/* Update the buffer with the new point */
            BPos = updateBuffer(BPos, numPosInBuffer, sPos, runCounterPos, t, (int)*(randCoinFlips + t), (int)*(randLocations + t));
            
			/* Did we just add a new point to the buffer ? */
            if(numPosInBuffer < sPos){
                numPosInBuffer++;
            }
        }	
		else if(y_t == -1){
			/* Does the buffer need to be refreshed ? */
			if(runCounterNeg > ENeg){
				/* Refresh the buffer and start a new epoch */
				for(bufferCounter = 0; bufferCounter < sNeg; bufferCounter++){
                    *(BNeg + bufferCounter) = -1;
                }
				numNegInBuffer = 0;
				runCounterNeg = 1;
            }
			/* Update the buffer with the new point */
            BNeg = updateBuffer(BNeg, numNegInBuffer, sNeg, runCounterNeg, t, (int)*(randCoinFlips + t), (int)*(randLocations + t));
            
			/* Did we just add a new point to the buffer ? */
            if(numNegInBuffer < sNeg){
                numNegInBuffer++;
            }
        }
    }
    
    free(BPos);
    free(BNeg);
    free(w);
    free(del_t);
    
    return wBar;
}

void mexFunction(int nlhs, mxArray *plhs[], int rhs, const mxArray *prhs[]){
	double *XTrain;                 /* Training data */
    double *yTrain;                 /* True labels */
    double C;                       /* C-SVM Value */
    int sPos;                       /* Size of positive buffer */
    int sNeg;                       /* Size of negative buffer */
    int EPos;                       /* Length of positive epoch */
    int ENeg;                       /* Length of negative epoch */
    double *randCoinFlips;          /* Random coin tosses to help us do things */
    double *randLocations;          /* Locations for buffer replacement */
    
    int numTrain;                   /* Number of training points */
    int d;                          /* Dimensionality of training points */
    
	double *wBar, *wBarTemp;        /* Final weight vector */
    
    double *temp;
    int dCounter,i,j,k;
    
	#define X_TRAIN	prhs[0]			/* Training data */
	#define Y_TRAIN	prhs[1]			/* True labels */
    #define C_IN    prhs[2]         /* C-SVM Value */
    #define	S_POS   prhs[3]         /* Size of positive buffer */
    #define	S_NEG   prhs[4]         /* Size of negative buffer */
    #define	E_POS   prhs[5]         /* Length of positive epoch */
    #define	E_NEG   prhs[6]         /* Length of negative epoch */
    #define RAND_COIN_FLIPS prhs[7] /* Random coin tosses to help us do things */
    #define RAND_LOCATIONS  prhs[8] /* Locations for buffer replacement */
    
	#define W_BAR	plhs[0]			/* Final weight vector */
            
    XTrain = mxGetPr(X_TRAIN);
    yTrain = mxGetPr(Y_TRAIN);
    
    temp = mxGetPr(C_IN);
    C = *(temp);
    
    temp = mxGetPr(S_POS);
    sPos = *(temp);
    
    temp = mxGetPr(S_NEG);
    sNeg = *(temp);
    
    temp = mxGetPr(E_POS);
    EPos = *(temp);
    
    temp = mxGetPr(E_NEG);
    ENeg = *(temp);
    
    randCoinFlips = mxGetPr(RAND_COIN_FLIPS);
    randLocations = mxGetPr(RAND_LOCATIONS);
    
	d = mxGetM(X_TRAIN);
    numTrain = mxGetN(X_TRAIN);
    
    W_BAR = mxCreateDoubleMatrix(d, 1, mxREAL);
    wBar = mxGetPr(W_BAR);
    
/*
    printf("Number of labels %d x %d",mxGetM(Y_TRAIN),mxGetN(Y_TRAIN));
*/
    
	if(numTrain != mxGetM(Y_TRAIN)){
		mexErrMsgTxt("Number of labels is not same as number of training points \n");
    }
    
	wBarTemp = doOAMGra(XTrain, yTrain, numTrain, d, C, sPos, sNeg, EPos, ENeg, randCoinFlips, randLocations);
    for(dCounter = 0; dCounter < d; dCounter++){
        *(wBar + dCounter) = *(wBarTemp + dCounter);
    }
    
    free(wBarTemp);
}