function [AUC] = OAMGra_RS_Mex(XTrain, yTrain, XTest, yTest, options)
	C = options.C;
	E = options.E;	% Epoch length
	s = options.s;	% Buffer size
    
	numTrain = length(yTrain);
    numPosTrain = sum(yTrain > 0);
    numNegTrain = sum(yTrain < 0);
    
	% Positive and negative buffers have the same size
	sPos = s;
	sNeg = s;
	
	% Positive and negative streams share the same epoch length
	% Do we have a single epoch ?
	if E == inf
		EPos = max(numPosTrain,numNegTrain) + 2;
		ENeg = max(numPosTrain,numNegTrain) + 2;
	else
		EPos = E;
		ENeg = E;
	end
    
    if E < s
        keyboard;
		error('Epoch length must be atleast the size of the buffers\n');
    end

	% Seed the random number generator
	timeNow = clock;
	msNow = round(timeNow(6)*1000);
	seedPrev = rng(msNow);
	
	% We need a random coin flip for each iteration
    temp = rand(1,numTrain);
	randPerm = randperm(numTrain);
	randCoinFlips = temp(randPerm);
	clear temp;
	
	% Seed the random number generator again
	timeNow = clock;
	msNow = round(timeNow(6)*1000);
	rng(msNow);
	
	% We also need a random replacement location at each iteration
    % Take care that we are generating locations for consumption in C
    temp = randi(s,1,numTrain) - 1;
	randPerm = randperm(numTrain);
	randLocations = temp(randPerm);
	clear temp
	
	% Restore the original seed
	rng(seedPrev);
	
	wBar = OAMGra_RS_C(XTrain,yTrain,C,sPos,sNeg,EPos,ENeg,randCoinFlips,randLocations);
	
	% Test time !!
	yPredicted = XTest'*wBar;
	correct = calculateAUC(yPredicted,yTest);

	numTestPos = sum(yTest > 0);
	numTestNeg = sum(yTest < 0);
	AUC = correct/(numTestPos*numTestNeg);
end