#include "mex.h"
#include <stdio.h>
#include <math.h>

int* updateBufferWithReplacement(int* B, int numInBuffer, int s, int t, int item, double* repopulationItems, double* numReplacements, double* replacementItems, int repopulationItemsUsed, int numReplacementsUsed, int replacementItemsUsed){
    int i;
    int* temp;
    
    /* If there is space just add it */
    if(numInBuffer < s){
        *(B + numInBuffer) = item;
    }/* Buffer got full in the last step - repopulate */
    else if((numInBuffer == s)&&(t == (s+1))){
        temp = (int*)malloc(t * sizeof(int));
        for(i = 0; i < s; i++){
            *(temp + i) = *(B + i);
        }
        *(temp + s) = item;
        for(i = 0; i < s; i++){
            *(B + i) = *(temp + (int)*(repopulationItems + repopulationItemsUsed + i));
        }
        free(temp);
    }/* Buffer got full a long time ago - just update */
    else if((numInBuffer == s)&&(t > s+1)){
        for(i = 0; i < (int)*(numReplacements + numReplacementsUsed); i++){
			*(B + (int)*(replacementItems + replacementItemsUsed + i)) = item;
        }
    }
    else{
        mexErrMsgTxt("Buffer in state of overflow !\n");
    }
    return B;
}

double* doOAMGra(double* XTrain, double* yTrain, int numTrain, int d, double C, int sPos, int sNeg, int EPos, int ENeg, double* repopulationItems, double* numReplacementsPos, double* numReplacementsNeg, double* replacementItems){
    int t, dCounter, bufferCounter, y_t, runCounterPos, runCounterNeg, updateCounter, numPosInBuffer, numNegInBuffer, numPointsInOtherBuffer, numLosses;
    double C_t, temp;
    double *x_t, *w, *wBar, *xBuf_t, *del_t;
    int *BPos, *BNeg, *otherBuffer;
    char msg[200];
    int junk, repopulationItemsUsed, numReplacementsPosUsed, numReplacementsNegUsed, replacementItemsUsed;
	
	/* We have not used any randomness till now */
	repopulationItemsUsed = 0;
	numReplacementsPosUsed = 0;
	numReplacementsNegUsed = 0;
	replacementItemsUsed = 0;
    
    /* Used to update the final weight vector after each update */
    updateCounter = 0;
    
    /* Run counters help us keep track of progress within an epoch */
	runCounterPos = 0;
	runCounterNeg = 0;
    
    /* Initialize the buffers to empty sets - buffers just contain the indices not the actual training points */
	BPos = (int*)malloc(sPos*sizeof(int));
	BNeg = (int*)malloc(sNeg*sizeof(int));
    
    for(bufferCounter = 0; bufferCounter < sPos; bufferCounter++){
        *(BPos + bufferCounter) = -1;
    }
    for(bufferCounter = 0; bufferCounter < sNeg; bufferCounter++){
        *(BNeg + bufferCounter) = -1;
    }
    
	numPosInBuffer = 0;
	numNegInBuffer = 0;
	
	C_t = 0;
    
    /* Intermediate weight vector */
	w = (double*)malloc(d*sizeof(double));
    
    /* Final weight vector */
    wBar = (double*)malloc(d*sizeof(double));
    
    /* Intermediate difference vector */
    del_t = (double*)malloc(d*sizeof(double));
    
	/* Initially both weight vectors are zero */
    for(dCounter = 0; dCounter < d; dCounter++){
        *(w + dCounter) = 0;
        *(wBar + dCounter) = 0;
    }
    
    for(t = 0; t < numTrain; t++){
        x_t = XTrain + d*t;
        y_t = (int)*(yTrain + t);
		C_t = C/sqrt(t+1);
        
        if(y_t == 1){
            runCounterPos++;
            otherBuffer = BNeg;
            numPointsInOtherBuffer = numNegInBuffer;
        }
        else if(y_t == -1){
            runCounterNeg++;
            otherBuffer = BPos;
            numPointsInOtherBuffer = numPosInBuffer;
        }
        else{
            junk = sprintf(&(msg[0]),"Bad label at location %d\n",t+1);
            mexErrMsgTxt(msg);
        }
        
        if(numPointsInOtherBuffer > 0){
			/* We will definitely make an update (even if a zero one) at this time step */
			updateCounter++;
			
			/* On how many points did we incur a loss */
			numLosses = 0;
			
			/* Set the step vector to zero */
			for(dCounter = 0; dCounter < d; dCounter++){
				*(del_t + dCounter) = 0;
			}
			
        	for(bufferCounter = 0; bufferCounter < numPointsInOtherBuffer; bufferCounter++){
				/* Get the buffer point */
				xBuf_t = XTrain + d*(*(otherBuffer + bufferCounter));
                
				/* Find out if this pair incurs any non zero hinge loss */
				/* i.e. if y_t * w_t(x_t - x_buf) < 1 */
				temp = 0;
                for(dCounter = 0; dCounter < d; dCounter++){
					temp += (*(w + dCounter))*(*(x_t + dCounter) - *(xBuf_t + dCounter));
                }
				temp *= y_t;
				
				/* If we have incured a loss then update the step vector */
				if(temp <= 1){
					numLosses++;	
					for(dCounter = 0; dCounter < d; dCounter++){
						*(del_t + dCounter) += *(xBuf_t + dCounter);
					}
				}
            }
			
			/* Update the intermediate weight vector if required */
			if(numLosses > 0){
				for(dCounter = 0; dCounter < d; dCounter++){
					*(w + dCounter) += C_t*y_t/numPointsInOtherBuffer*(numLosses*(*(x_t + dCounter)) - *(del_t + dCounter));
				}
			}
			
			/* Update the final weight vector */
			for(dCounter = 0; dCounter < d; dCounter++){
				*(wBar + dCounter) = ((*(wBar + dCounter))*(updateCounter - 1) + *(w + dCounter))/updateCounter;
			}
        }
                
        /* Update the respective buffers */
		if(y_t == +1){
			/* Does the buffer need to be refreshed ? */
			if(runCounterPos > EPos){
				/* Refresh the buffer and start a new epoch */
                for(bufferCounter = 0; bufferCounter < sPos; bufferCounter++){
                    *(BPos + bufferCounter) = -1;
                }
				numPosInBuffer = 0;
				runCounterPos = 1;
            }
			/* Update the buffer with the new point */
            BPos = updateBufferWithReplacement(BPos, numPosInBuffer, sPos, runCounterPos, t, repopulationItems, numReplacementsPos, replacementItems, repopulationItemsUsed, numReplacementsPosUsed, replacementItemsUsed);
            
            /* Did we just add a new point to the buffer ? */
            if(numPosInBuffer < sPos){
                numPosInBuffer++;
            }/* If we used up some randomness, move the pointers forward */
            else if(runCounterPos == sPos + 1){
                repopulationItemsUsed += sPos;
            }
            else if(runCounterPos > sPos + 1){
				numReplacementsPosUsed += 1;
                replacementItemsUsed += sPos;
            }
            
        }
		else if(y_t == -1){
			/* Does the buffer need to be refreshed ? */
			if(runCounterNeg > ENeg){
				/* Refresh the buffer and start a new epoch */
				for(bufferCounter = 0; bufferCounter < sNeg; bufferCounter++){
                    *(BNeg + bufferCounter) = -1;
                }
				numNegInBuffer = 0;
				runCounterNeg = 1;
            }
			/* Update the buffer with the new point */
            BNeg = updateBufferWithReplacement(BNeg, numNegInBuffer, sNeg, runCounterNeg, t, repopulationItems, numReplacementsNeg, replacementItems, repopulationItemsUsed, numReplacementsNegUsed, replacementItemsUsed);
            
            /* Did we just add a new point to the buffer ? */
            if(numNegInBuffer < sNeg){
                numNegInBuffer++;
            }/* If we used up some randomness, move the pointers forward */
            else if(runCounterNeg == sNeg + 1){
                repopulationItemsUsed += sNeg;
            }
            else if(runCounterNeg > sNeg + 1){
                numReplacementsNegUsed += 1;
                replacementItemsUsed += sNeg;
            }
        }
    }
    
    free(BPos);
    free(BNeg);
    free(w);
    free(del_t);
    
    return wBar;
}

void mexFunction(int nlhs, mxArray *plhs[], int rhs, const mxArray *prhs[]){
	double *XTrain;                 /* Training data */
    double *yTrain;                 /* True labels */
    double C;                       /* C-SVM Value */
    int sPos;                       /* Size of positive buffer */
    int sNeg;                       /* Size of negative buffer */
    int EPos;                       /* Length of positive epoch */
    int ENeg;                       /* Length of negative epoch */
    double *repopulationItems;      /* Items for buffer updates at the first instance of overflow */
	double *numReplacementsPos;		/* How many items to replace in positive buffer in normal iterations */
	double *numReplacementsNeg;		/* How many items to replace in negative buffer in normal iterations */
	double *replacementItems;		/* Which items to replace in normal iterations */
    
    int numTrain;                   /* Number of training points */
    int d;                          /* Dimensionality of training points */
    
	double *wBar, *wBarTemp;        /* Final weight vector */
    
    double *temp;
    int dCounter,i,j,k;
    
	#define X_TRAIN	prhs[0]			/* Training data */
	#define Y_TRAIN	prhs[1]			/* True labels */
    #define C_IN    prhs[2]         /* C-SVM Value */
    #define	S_POS   prhs[3]         /* Size of positive buffer */
    #define	S_NEG   prhs[4]         /* Size of negative buffer */
    #define	E_POS   prhs[5]         /* Length of positive epoch */
    #define	E_NEG   prhs[6]         /* Length of negative epoch */
    #define REPOP_ITEMS  prhs[7]    /* Items for buffer updates at the first instance of overflow */
	#define NUMREP_POS	prhs[8]		/* How many items to replace in positive buffer in normal iterations */
	#define NUMREP_NEG	prhs[9]		/* How many items to replace in negative buffer in normal iterations */
	#define REPL_ITEMS	prhs[10]	/* Which items to replace in normal iterations */
    
	#define W_BAR	plhs[0]			/* Final weight vector */
            
    XTrain = mxGetPr(X_TRAIN);
    yTrain = mxGetPr(Y_TRAIN);
    
    temp = mxGetPr(C_IN);
    C = *(temp);
    
    temp = mxGetPr(S_POS);
    sPos = *(temp);
    
    temp = mxGetPr(S_NEG);
    sNeg = *(temp);
    
    temp = mxGetPr(E_POS);
    EPos = *(temp);
    
    temp = mxGetPr(E_NEG);
    ENeg = *(temp);
    
    repopulationItems = mxGetPr(REPOP_ITEMS);
    numReplacementsPos = mxGetPr(NUMREP_POS);
	numReplacementsNeg = mxGetPr(NUMREP_NEG);
	replacementItems = mxGetPr(REPL_ITEMS);
	
	d = mxGetM(X_TRAIN);
    numTrain = mxGetN(X_TRAIN);
    
    W_BAR = mxCreateDoubleMatrix(d, 1, mxREAL);
    wBar = mxGetPr(W_BAR);
    
	if(numTrain != mxGetM(Y_TRAIN)){
		mexErrMsgTxt("Number of labels is not same as number of training points \n");
    }
    
	wBarTemp = doOAMGra(XTrain, yTrain, numTrain, d, C, sPos, sNeg, EPos, ENeg, repopulationItems, numReplacementsPos, numReplacementsNeg, replacementItems);
	
    for(dCounter = 0; dCounter < d; dCounter++){
        *(wBar + dCounter) = *(wBarTemp + dCounter);
    }
    
    free(wBarTemp);
}