function [AUC] = OAMGra_mRS_Mex(XTrain, yTrain, XTest, yTest, options)
	C = options.C;
	E = options.E;	% Epoch length
	s = options.s;	% Buffer size
    
    numPosTrain = sum(yTrain > 0);
    numNegTrain = sum(yTrain < 0);
    
	% Positive and negative buffers have the same size
	sPos = s;
	sNeg = s;
	
	% Positive and negative streams share the same epoch length
	% Do we have a single epoch ?
	if E == inf
		EPos = max(numPosTrain,numNegTrain) + 2;
		ENeg = max(numPosTrain,numNegTrain) + 2;
	else
		EPos = E;
		ENeg = E;
	end
    
    if E < s
        keyboard;
		error('Epoch length must be atleast the size of the buffers\n');
    end

    % How many epochs do we have - one repopulation per epoch plus some extra for good measure :D
	% Do we have a single epoch ?
	if E == inf
		numRepopulationInstances = 4;
	else
		numRepopulationInstances = ceil(numPosTrain/EPos) + ceil(numNegTrain/ENeg) + 2;
	end
        
    % We have E-s-1 normal updates per epoch plus some extra for good measure :D
	% Do we have a single epoch ?
	if E == inf
		numUpdateInstancesPos = max(numPosTrain - sPos - 1,0) + 1;
		numUpdateInstancesNeg = max(numNegTrain - sNeg - 1,0) + 1;
	else
		numUpdateInstancesPos = ceil(numPosTrain/EPos)*max(EPos - sPos - 1,0) + 1;
		numUpdateInstancesNeg = ceil(numNegTrain/ENeg)*max(ENeg - sNeg - 1,0) + 1;
	end
	
	% Seed the random number generator
	timeNow = clock;
	msNow = round(timeNow(6)*1000);
	seedPrev = rng(msNow);
	
    % We need a binomial random variable at each update instance to tell us how many points are going out of the buffer
	if numUpdateInstancesPos > 0
		temp = sPos + 2:numPosTrain;
		numReplacePos = binornd(sPos ,1./temp);
	else
		numReplacePos = 0;
	end
	
	if numUpdateInstancesNeg > 0
		temp = sNeg + 2:numNegTrain;
		numReplaceNeg = binornd(sNeg ,1./temp);
	else
		numReplaceNeg = 0;
	end
	
	% Seed the random number generator again
	timeNow = clock;
	msNow = round(timeNow(6)*1000);
	rng(msNow);
	
	% We also need the items that would be replaced at each normal update step.
	% In order to simplify things, we are just sending a random permutation for each step. Only the first few positions will be replaced.
	% Take care that we are generating locations for consumption in C
	numUpdateInstances = numUpdateInstancesPos + numUpdateInstancesNeg;
	sMax = max(sPos,sNeg);
	if numUpdateInstances > 0
		replacementItems = zeros(sMax,numUpdateInstances);
		for i = 1:numUpdateInstances
			replacementItems(:,i) = randperm(sMax) - 1;
		end			
	else
		replacementItems = zeros(sMax,1);
	end
	
	% Seed the random number generator again
	timeNow = clock;
	msNow = round(timeNow(6)*1000);
	rng(msNow);
	
    % We also need s+1 random replacement locations for at each iteration at the (s+1)^th time step of each epoch
    % Take care that we are generating locations for consumption in C
	
	% If we dont have too many repopulation instances
	if numRepopulationInstances < 10;
		repopulationItems = zeros(sMax,numRepopulationInstances);
		for i = 1:sMax
			for j = i:numRepopulationInstances
				temp = randperm(sMax+1);
				repopulationItems(i,j) = temp(1) - 1;
				clear temp
			end
		end
	else
		repopulationItems = randi(sMax+1,sMax,numRepopulationInstances) - 1;
	end
	
	% Restore the original seed
	rng(seedPrev);
	
	wBar = OAMGra_mRS_C(XTrain,yTrain,C,sPos,sNeg,EPos,ENeg,repopulationItems,numReplacePos,numReplaceNeg,replacementItems);
	
	% Test time !!
	yPredicted = XTest'*wBar;
	correct = calculateAUC(yPredicted,yTest);

	numTestPos = sum(yTest > 0);
	numTestNeg = sum(yTest < 0);
	AUC = correct/(numTestPos*numTestNeg);
end