function [CBest] = doCVForC(XTrain, yTrain, options, foldIds, numFolds, trainingRoutine)
    if length(yTrain) ~= length(foldIds)
        error('Mismatch in number of training points and foldIDs');
    end
    
    if ~isequal(unique(foldIds)', 1:numFolds)
        error('Too many folds or some folds missing');
    end
    
	CVOptions = options;

	CBest = -inf;
	AUCBest = 0;
	AUCAcrossFolds = zeros(numFolds,1);

	for i = -10:2:10
		CVOptions.C = 2^i;

		% Try out this value of C
		for j = 1:numFolds
			XTrainFold = XTrain(:,foldIds ~= j);
			yTrainFold = yTrain(foldIds ~= j);
			XTestFold = XTrain(:,foldIds == j);
			yTestFold = yTrain(foldIds == j);
			
			AUCAcrossFolds(j) = trainingRoutine(XTrainFold, yTrainFold, XTestFold, yTestFold, CVOptions);
		end
		
		temp = mean(AUCAcrossFolds);
		if temp > AUCBest,
			AUCBest = temp;
			CBest = 2^i;
		end
	end

	fprintf(1,'OAM: The best AUC value is %d\n', AUCBest);
	fprintf(1,'OAM: The best penalty parameter is %d\n', CBest);
end