function doZhaoExperiment_InfiniteEpoch
    splitOptions = load('datasets/splitOptions.mat');
    splitOptions = splitOptions.splitOptions;
    
    numFolds = splitOptions.numFolds;
    numSplits = splitOptions.numSplits;
	
	% No epochs so set epoch length to some ridiculously high value
	options.E = inf;
    
	datasets = {'diabetes','sonar'};
	numDatasets = length(datasets);
    
    globalSValues = [1:2:11 20:20:100 200:100:1000];
    
    baseLearner = @OAMGra_RS_Wrapper;
	
	% Too many test points can slow down AUC calculation
	maxNumTest = 50000;
    
    for datasetCounter = 1:numDatasets
        X = load(['datasets/' datasets{datasetCounter} '.X.mat']);
        X = X.X;
        y = load(['datasets/' datasets{datasetCounter} '.y.mat']);
        y = y.y;
        splits = load(['datasets/' datasets{datasetCounter} '.splits.mat']);
        splits = splits.splits;

        sValues = [globalSValues(globalSValues < splits.numTraining) splits.numTraining];
		sValues = sValues(sValues <= 1000);
        numSValues = length(sValues);
		
		save(sprintf('Res_Gra_RS_InfiniteEpoch/sValues_Gra_RS_InfiniteEpoch_%s.mat',datasets{datasetCounter}),'sValues');

        AUCValues = zeros(numSValues, numSplits);
        CValues = zeros(numSValues, numSplits);
		
		numTestThisDataset = min(maxNumTest,length(y)-splits.numTraining);
		
		for splitCounter = 1 : numSplits
            % Normalize data
            XThisSplit = bsxfun(@plus,full(X),-splits.means(:,splitCounter));
            XThisSplit = bsxfun(@rdivide,XThisSplit,splits.stds(:,splitCounter));

            % Apply split permutation
            XThisSplit = XThisSplit(:,splits.IDs(:,splitCounter));
            yThisSplit = full(y(splits.IDs(:,splitCounter)));

            % Get the training and test splits
            XTrainingThisSplit = XThisSplit(:,1:splits.numTraining);
            yTrainingThisSplit = yThisSplit(1:splits.numTraining);
            XTestThisSplit = XThisSplit(:,splits.numTraining+1:splits.numTraining+numTestThisDataset);
            yTestThisSplit = yThisSplit(splits.numTraining+1:splits.numTraining+numTestThisDataset);

            % Vary buffer sizes
            for sValueCounter = 1 : numSValues
				fprintf('Dataset %s Split number %d Buffer counter %d of %d Buffer size %d \n',datasets{datasetCounter},splitCounter,sValueCounter,numSValues,sValues(sValueCounter));
				
				% Get a good value of C
				options.s = sValues(sValueCounter);
				CThisSplit = doCVForC(XThisSplit(:,1:splits.numValidating),yThisSplit(1:splits.numValidating),options,splits.foldIDs(:,splitCounter),numFolds,baseLearner);
				
				CValues(sValueCounter,splitCounter) = CThisSplit;

				options.C = CThisSplit;
				AUCThisSplit = baseLearner(XTrainingThisSplit, yTrainingThisSplit, XTestThisSplit, yTestThisSplit, options);

				AUCValues(sValueCounter,splitCounter) = AUCThisSplit;

				save(sprintf('Res_Gra_RS_InfiniteEpoch/AUCValues_Gra_RS_InfiniteEpoch_%s.mat',datasets{datasetCounter}),'AUCValues');
				save(sprintf('Res_Gra_RS_InfiniteEpoch/CValues_Gra_RS_InfiniteEpoch_%s.mat',datasets{datasetCounter}),'CValues');
            end
        end
    end
end